@@ -37,6 +37,9 @@ sub BUILD {
   
   $self->add_plugin('apptree-serverfilter');
   
+  # Example feature:
+  $self->add_plugin('rapifs-dblclick-download');
+  
 }
 
 around call_fetch_nodes => sub {
@@ -127,27 +130,31 @@ sub _get_render_content_type {
 }
 
 sub _apply_node_view_url {
-  my ($self, $Node, $mount) = @_;
+  my ($self, $Node, $mount, $for_ext) = @_;
   
   my $enc_path = $Node->path && $Node->path ne '/'
     ? $self->b64_encode(join('/',$mount,$Node->path))
     : $self->b64_encode($mount);
     
-  $Node->view_url( $self->suburl($enc_path) );
+  $Node->view_url( $for_ext ? $self->local_url($enc_path) : $self->suburl($enc_path) );
   
   $enc_path
 }
 
 sub _apply_node_open_url {
-  my ($self, $Node, $mount) = @_;
+  my ($self, $Node, $mount, $for_ext) = @_;
   
-  my $enc_path = $self->_apply_node_view_url($Node,$mount);
+  my $enc_path = $self->_apply_node_view_url($Node,$mount,$for_ext);
   
   if ($Node->readable_file) {
     $Node->download_url( join('',$Node->view_url,'?method=download'));
     $Node->open_url( join('',$Node->view_url,'?method=open')) if (
          $Node->bytes < $self->max_render_bytes
       && $self->_get_render_content_type( $Node )
+    );
+    $Node->source_url( join('',$Node->view_url,'?method=source')) if (
+         $Node->bytes < $self->max_render_bytes
+      && $Node->code_language
     )
   }
   
@@ -260,6 +267,14 @@ sub _folder_up_treenode {
 around 'content' => sub {
   my ($orig,$self,@args) = @_;
   
+  # This extra feature has been added for example purposes, see rapifs.js for
+  # what this mode activates and does
+  if(my $special = $self->c->req->params->{special}) {
+    $self->apply_extconfig(  
+      dblclick_download => 1
+    ) if($special eq 'dblclick_download');
+  }
+  
   if (my $Node = $self->Node_from_local_args) {
   
     my $mount    = $Node->driver->name;
@@ -327,7 +342,7 @@ around 'content' => sub {
       }
       elsif($meth eq 'view') {
       
-        $self->_apply_node_open_url($Node,$mount);
+        $self->_apply_node_open_url($Node,$mount,1);
         $self->_apply_node_open_url($Node->parent,$mount);
 
         $c->stash->{template}   = 'fileview.html';
@@ -335,6 +350,12 @@ around 'content' => sub {
 
         return $c->detach( $c->view('RapidApp::Template') );
       }
+      elsif($meth eq 'source') {
+        $c->stash->{template}   = 'sourceview.html';
+        $c->stash->{RapiFsFile} = $Node;
+
+        return $c->detach( $c->view('RapidApp::Template') );
+      }
       else {
         die usererr "No such method '$meth'";
       
@@ -371,6 +392,7 @@ around 'auto_hashnav_redirect_current' => sub {
     # Or in the case of a supplied known method param
     || $uri_query eq 'method=open'
     || $uri_query eq 'method=download'
+    || $uri_query eq 'method=source'
   )
 };
 
